--[[
Author: Blythe
Filename: ChatAlert.lua
Date: 2014-5-23
Version: 2.5
]]--

ChatAlert = {}
ChatAlert.name = "ChatAlert"
local savedVariables

setsound = setmetatable({}, { __index=function(tbl, index) return function(val) SetSound(index, val) end end })
settext = setmetatable({}, { __index=function(tbl, index) return function(val) SetText(index, val) end end })
getsound = setmetatable({}, { __index=function(tbl, index) return function() return GetSound(index) end end })
gettext = setmetatable({}, { __index=function(tbl, index) return function() return GetText(index) end end })

local defaults = {			-- default settings for saved variables
	silenceMe = 1,
	soundTell = "New_Mail",
	soundName = "New_Mail",
	soundGroup = "New_Mail",
	soundContacts = "New_Mail",
	soundOnline = "Overland_Boss_Kill",
	soundOffline = "Quest_Abandon",
	soundG1 = "No_Sound",
	soundG2 = "No_Sound",
	soundG3 = "No_Sound",
	soundG4 = "No_Sound",
	soundG5 = "No_Sound",
	soundZone = "No_Sound",
	soundSay = "No_Sound",
	soundYell = "No_Sound",
	soundText1 = "No_Sound",
	soundText2 = "No_Sound",
	soundText3 = "No_Sound",
	soundText4 = "No_Sound",
	soundText5 = "No_Sound",
	text1 = "",
	text2 = "",
	text3 = "",
	text4 = "",
	text5 = "",
}

function ChatAlert.init(eventCode, addOnName)
    if addOnName ~= "ChatAlert" then 
		return 
	end
	
	savedVariables = ZO_SavedVars:New("ChatAlert_SavedVariables", 1, nil, defaults)
	
	myName = GetUnitName('player') or 'Unknown'
    myAccount = GetDisplayName() or 'Unknown'
	
	ChatAlert.Options()
	
	EVENT_MANAGER:RegisterForEvent("ChatAlert", EVENT_FRIEND_PLAYER_STATUS_CHANGED, ChatAlert.friend_status)
	EVENT_MANAGER:RegisterForEvent("ChatAlert", EVENT_CHAT_MESSAGE_CHANNEL, ChatAlert.handleChat)
end

function ChatAlert.friend_status(eventID, username, state, previous_state)
	if state == 1 then
		PlaySound(savedVariables.soundOffline)
	elseif state == 4 then
		PlaySound(savedVariables.soundOnline)
	end
end

function ChatAlert.handleChat(event, messageType, fromName, text)
    local ChannelInfo = ZO_ChatSystem_GetChannelInfo()
	local channelInfo = ChannelInfo[messageType]

	if channelInfo and channelInfo.format and not (savedVariables.silenceMe and (string.find(fromName, myName) or string.find(fromName, myAccount))) then

		if savedVariables.text1 ~= "" and string.find(string.lower(text), string.lower(savedVariables.text1)) and savedVariables.soundText1 ~= "No_Sound" then
			PlaySound(savedVariables.soundText1)
		elseif savedVariables.text2 ~= "" and string.find(string.lower(text), string.lower(savedVariables.text2)) and savedVariables.soundText2 ~= "No_Sound" then
			PlaySound(savedVariables.soundText2)
		elseif savedVariables.text3 ~= "" and string.find(string.lower(text), string.lower(savedVariables.text3)) and savedVariables.soundText3 ~= "No_Sound" then
			PlaySound(savedVariables.soundText3)
		elseif savedVariables.text4 ~= "" and string.find(string.lower(text), string.lower(savedVariables.text4)) and savedVariables.soundText4 ~= "No_Sound" then
			PlaySound(savedVariables.soundText4)
		elseif savedVariables.text5 ~= "" and string.find(string.lower(text), string.lower(savedVariables.text5)) and savedVariables.soundText5 ~= "No_Sound" then
			PlaySound(savedVariables.soundText5)
		elseif string.find(string.lower(text), string.lower(myName)) or string.find(string.lower(text), string.sub(string.lower(myAccount),2)) and savedVariables.soundName ~= "No_Sound" then
			PlaySound(savedVariables.soundName)
		elseif channelInfo.id == 2 and savedVariables.soundTell ~= "No_Sound" then
			PlaySound(savedVariables.soundTell)
		elseif channelInfo.id == 3 and savedVariables.soundGroup ~= "No_Sound" then
			PlaySound(savedVariables.soundGroup)
		elseif IsFriend(fromName) and savedVariables.soundContacts ~= "No_Sound" then
			PlaySound(savedVariables.soundContacts)
		elseif channelInfo.id == 12 and savedVariables.soundG1 ~= "No_Sound" then
			PlaySound(savedVariables.soundG1)
		elseif channelInfo.id == 13 and savedVariables.soundG2 ~= "No_Sound" then
			PlaySound(savedVariables.soundG2)
		elseif channelInfo.id == 14 and savedVariables.soundG3 ~= "No_Sound" then
			PlaySound(savedVariables.soundG3)
		elseif channelInfo.id == 15 and savedVariables.soundG4 ~= "No_Sound" then
			PlaySound(savedVariables.soundG4)
		elseif channelInfo.id == 16 and savedVariables.soundG5 ~= "No_Sound" then
			PlaySound(savedVariables.soundG5)
		elseif channelInfo.id == 1 and savedVariables.soundYell ~= "No_Sound" then
			PlaySound(savedVariables.soundYell)
		elseif channelInfo.id == 0 and savedVariables.soundSay ~= "No_Sound" then
			PlaySound(savedVariables.soundSay)
		elseif channelInfo.id == 31 and savedVariables.soundZone ~= "No_Sound" then
			PlaySound(savedVariables.soundZone)
		end

	end
end

function SetSound(key, newSound)
	PlaySound(newSound)
	savedVariables["sound"..key] = newSound
end

function SetText(key, newString)
	savedVariables[key] = newString
end

function GetSound(key)
	return savedVariables["sound"..key]
end

function GetText(key)
	return savedVariables[key]
end

function ChatAlert.Options()
	local alert_sounds = {
	"No_Sound",
	"Ability_Failed",
	"Book_Acquired",
	"Click",
	"Click_Negative",
	"Click_Positive",
	"Codex_Close",
	"Codex_Open",
	"Dialog_Show",
	"General_Alert_Error",
	"Lockpicking_lockpick_contact",
	"Lockpicking_success",
	"Lockpicking_unlocked",
	"Mail_Sent",
	"New_Mail",
	"Note_Open",
	"Overland_Boss_Kill",
	"Quest_Abandon",
	"Quest_Accept",
	"Quest_StepFailed",
	"Radial_Menu_Mouseover",
	"SkillLine_Added",
	"SkillLine_Leveled",
	"SkillType_World",
	"Stats_Purchase"	
	}
	
	--[[ I wanted to use the below to list all sounds, but the list is just too unwieldy without a scrollbar on the dropdown.  Someone else might find use in the function, though...
	
	local i=1
	for key,name in pairs(SOUNDS) do
		alert_sounds[i] = name
		i = (i + 1) %2
	end
	table.sort(alert_sounds)
	
	]]--

	local LAM = LibStub("LibAddonMenu-1.0")
    local ChatAlertPanel = LAM:CreateControlPanel("CHATALERT_OPTIONS", "ChatAlert")
	
	LAM:AddHeader(ChatAlertPanel, "CHATALERT_OPTIONS_HEADER_SOUNDS", "Alert sounds")
	
	LAM:AddDescription(ChatAlertPanel, "CHATALERT_OPTIONS_DESCRIPTION_SOUNDS", "If you would like different sounds added to the lists, send a mail to @Blythe.", "")
	
	LAM:AddCheckbox(ChatAlertPanel, "CHATALERT_OPTIONS_SILENCEME", "Silence My Own Messages", "Set to \"on\" to prevent your own messages from triggering any of the below sounds", function () return savedVariables.silenceMe end, function (value) savedVariables.silenceMe = value end, warning, warningText)
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_TELL", "Whispers/Tells", "Choose a sound to use when someone sends you a whisper/tell", alert_sounds, getsound.Tell, setsound.Tell, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_NAME", "@Account/Name Said in Chat", "Choose a sound to use when someone ways your name (not case sensitive)", alert_sounds, getsound.Name, setsound.Name, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_GROUP", "Group", "Choose a sound to use when your group talks in chat", alert_sounds, getsound.Group, setsound.Group, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_CONTACTS", "Contacts", "Choose a sound to use when contacts talk in chat", alert_sounds, getsound.Contacts, setsound.Contacts, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_ONLINE", "Contact Online", "Choose a sound to use when a contact comes online", alert_sounds, getsound.Online, setsound.Online, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_OFFLINE", "Contact Offline", "Choose a sound to use when a contact goes offline", alert_sounds, getsound.Offline, setsound.Offline, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_G1", "Guild 1", "Choose a sound to use when Guild 1 talks in chat", alert_sounds, getsound.G1, setsound.G1,	FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_G2", "Guild 2", "Choose a sound to use when Guild 2 talks in chat", alert_sounds, getsound.G2, setsound.G2,	FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_G3", "Guild 3", "Choose a sound to use when Guild 3 talks in chat", alert_sounds, getsound.G3, setsound.G3, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_G4", "Guild 4", "Choose a sound to use when Guild 4 talks in chat", alert_sounds, getsound.G4, setsound.G4, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_G5", "Guild 5", "Choose a sound to use when Guild 5 talks in chat", alert_sounds, getsound.G5, setsound.G5, FALSE, "")

	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_YELL", "Yell", "Choose a sound to use when someone yells in chat", alert_sounds, getsound.Yell, setsound.Yell, FALSE, "")
	
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_SAY", "Say", "Choose a sound to use for /say messages", alert_sounds, getsound.Say, setsound.Say, FALSE, "")
		
	LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_ZONE", "Zone", "Choose a sound for zone chat", alert_sounds, getsound.Zone, setsound.Zone, 1, "This will be very noisy!")
		
	LAM:AddHeader(ChatAlertPanel, "CHATALERT_OPTIONS_HEADER_WORDSPHRASES", "Alert words/phrases")
	
	LAM:AddDescription(ChatAlertPanel, "CHATALERT_OPTIONS_DESCRIPTION_WORDSPHRASES", "Not case sensitive.", "")
	
	for i=1,5,1 do
		LAM:AddEditBox(ChatAlertPanel, "CHATALERT_OPTIONS_TEXT"..i, "Word/phrase "..i, "The sound below will play when this word/phrase comes up in chat", FALSE, gettext["text"..i], settext["text"..i], FALSE, "")
		
		LAM:AddDropdown(ChatAlertPanel, "CHATALERT_OPTIONS_SOUND_TEXT"..i, "", "Choose a sound to use for word/phrase "..i, alert_sounds, getsound["Text"..i], setsound["Text"..i], FALSE, "")
	end


end

EVENT_MANAGER:RegisterForEvent("ChatAlert", EVENT_ADD_ON_LOADED, ChatAlert.init)